package com.conflux.web.controller.collect.controller;


import com.conflux.common.annotation.Log;
import com.conflux.common.core.controller.BaseController;
import com.conflux.common.core.domain.AjaxResult;
import com.conflux.common.core.page.TableDataInfo;
import com.conflux.common.enums.BusinessType;
import com.conflux.common.utils.poi.ExcelUtil;
import com.conflux.web.controller.collect.domain.CollectConfig;
import com.conflux.web.controller.collect.service.ICollectConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * confluxController
 * 
 * @author conflux
 * @date 2022-05-31
 */
@RestController
@RequestMapping("/conflux/collect")
public class CollectConfigController extends BaseController
{
    @Autowired
    private ICollectConfigService collectConfigService;

    /**
     * 查询conflux列表
     */
    @PreAuthorize("@ss.hasPermi('conflux:collect:list')")
    @GetMapping("/list")
    public TableDataInfo list(CollectConfig collectConfig)
    {
        startPage();
        List<CollectConfig> list = collectConfigService.selectCollectConfigList(collectConfig);
        return getDataTable(list);
    }

    /**
     * 导出conflux列表
     */
    @PreAuthorize("@ss.hasPermi('conflux:collect:export')")
    @Log(title = "conflux", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CollectConfig collectConfig)
    {
        List<CollectConfig> list = collectConfigService.selectCollectConfigList(collectConfig);
        ExcelUtil<CollectConfig> util = new ExcelUtil<CollectConfig>(CollectConfig.class);
        util.exportExcel(response, list, "conflux数据");
    }

    /**
     * 获取conflux详细信息
     */
    @PreAuthorize("@ss.hasPermi('conflux:collect:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(collectConfigService.selectCollectConfigById(id));
    }

    /**
     * 新增conflux
     */
    @PreAuthorize("@ss.hasPermi('conflux:collect:add')")
    @Log(title = "conflux", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CollectConfig collectConfig)
    {
        return toAjax(collectConfigService.insertCollectConfig(collectConfig));
    }

    /**
     * 修改conflux
     */
    @PreAuthorize("@ss.hasPermi('conflux:collect:edit')")
    @Log(title = "conflux", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CollectConfig collectConfig)
    {
        return toAjax(collectConfigService.updateCollectConfig(collectConfig));
    }

    /**
     * 删除conflux
     */
    @PreAuthorize("@ss.hasPermi('conflux:collect:remove')")
    @Log(title = "conflux", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(collectConfigService.deleteCollectConfigByIds(ids));
    }
    @PutMapping("/changeStatus")
    public AjaxResult changeStatus(@RequestBody CollectConfig collectConfig){
        return toAjax(collectConfigService.changeStatus(collectConfig));
    }
}
