package com.conflux.web.controller.collect.service.impl;


import com.conflux.common.utils.DateUtils;
import com.conflux.web.controller.collect.domain.CollectConfig;
import com.conflux.web.controller.collect.mapper.CollectConfigMapper;
import com.conflux.web.controller.collect.service.ICollectConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * confluxService业务层处理
 *
 * @author conflux
 * @date 2022-05-31
 */
@Service
public class CollectConfigServiceImpl implements ICollectConfigService {
    @Autowired
    private CollectConfigMapper collectConfigMapper;

    /**
     * 查询conflux
     *
     * @param id conflux主键
     * @return conflux
     */
    @Override
    public CollectConfig selectCollectConfigById(Long id) {
        return collectConfigMapper.selectCollectConfigById(id);
    }

    /**
     * 查询conflux列表
     *
     * @param collectConfig conflux
     * @return conflux
     */
    @Override
    public List<CollectConfig> selectCollectConfigList(CollectConfig collectConfig) {
        return collectConfigMapper.selectCollectConfigList(collectConfig);
    }

    /**
     * 新增conflux
     *
     * @param collectConfig conflux
     * @return 结果
     */
    @Override
    public int insertCollectConfig(CollectConfig collectConfig) {
        collectConfig.setCreateTime(System.currentTimeMillis());

        return collectConfigMapper.insertCollectConfig(collectConfig);
    }

    /**
     * 修改conflux
     *
     * @param collectConfig conflux
     * @return 结果
     */
    @Override
    public int updateCollectConfig(CollectConfig collectConfig) {
        collectConfig.setUpdateTime(System.currentTimeMillis());
        return collectConfigMapper.updateCollectConfig(collectConfig);
    }

    /**
     * 批量删除conflux
     *
     * @param ids 需要删除的conflux主键
     * @return 结果
     */
    @Override
    public int deleteCollectConfigByIds(Long[] ids) {
        return collectConfigMapper.deleteCollectConfigByIds(ids);
    }

    /**
     * 删除conflux信息
     *
     * @param id conflux主键
     * @return 结果
     */
    @Override
    public int deleteCollectConfigById(Long id) {
        return collectConfigMapper.deleteCollectConfigById(id);
    }

    @Override
    @Transactional
    public int changeStatus(CollectConfig collectConfig) {
        collectConfigMapper.updateCollectConfig(collectConfig);
        return collectConfigMapper.updateStatus(collectConfig.getId());//其他禁用
    }

    @Override
    public CollectConfig selectCollectConfigByStatus() {
        return collectConfigMapper.selectCollectConfigByStatus();
    }
}
