package com.conflux.web.controller.contract.controller;


import com.conflux.common.annotation.Log;
import com.conflux.common.core.controller.BaseController;
import com.conflux.common.core.domain.AjaxResult;
import com.conflux.common.core.page.TableDataInfo;
import com.conflux.common.enums.BusinessType;
import com.conflux.common.utils.poi.ExcelUtil;
import com.conflux.web.controller.contract.domain.ContractConfig;
import com.conflux.web.controller.contract.service.IContractConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 合约私钥配置Controller
 * 
 * @author conflux
 * @date 2022-05-31
 */
@RestController
@RequestMapping("/conflux/contract")
public class ContractConfigController extends BaseController
{
    @Autowired
    private IContractConfigService contractConfigService;

    /**
     * 查询合约私钥配置列表
     */
    @PreAuthorize("@ss.hasPermi('conflux:contract:list')")
    @GetMapping("/list")
    public TableDataInfo list(ContractConfig contractConfig)
    {
        startPage();
        List<ContractConfig> list = contractConfigService.selectContractConfigList(contractConfig);
        return getDataTable(list);
    }

    /**
     * 导出合约私钥配置列表
     */
    @PreAuthorize("@ss.hasPermi('conflux:contract:export')")
    @Log(title = "合约私钥配置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ContractConfig contractConfig)
    {
        List<ContractConfig> list = contractConfigService.selectContractConfigList(contractConfig);
        ExcelUtil<ContractConfig> util = new ExcelUtil<ContractConfig>(ContractConfig.class);
        util.exportExcel(response, list, "合约私钥配置数据");
    }

    /**
     * 获取合约私钥配置详细信息
     */
    @PreAuthorize("@ss.hasPermi('conflux:contract:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(contractConfigService.selectContractConfigById(id));
    }

    /**
     * 新增合约私钥配置
     */
    @PreAuthorize("@ss.hasPermi('conflux:contract:add')")
    @Log(title = "合约私钥配置", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ContractConfig contractConfig)
    {
        return toAjax(contractConfigService.insertContractConfig(contractConfig));
    }

    /**
     * 修改合约私钥配置
     */
    @PreAuthorize("@ss.hasPermi('conflux:contract:edit')")
    @Log(title = "合约私钥配置", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ContractConfig contractConfig)
    {
        return toAjax(contractConfigService.updateContractConfig(contractConfig));
    }

    /**
     * 删除合约私钥配置
     */
    @PreAuthorize("@ss.hasPermi('conflux:contract:remove')")
    @Log(title = "合约私钥配置", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(contractConfigService.deleteContractConfigByIds(ids));
    }
}
