package com.conflux.web.controller.nft.controller;


import com.conflux.common.annotation.Log;
import com.conflux.common.core.controller.BaseController;
import com.conflux.common.core.domain.AjaxResult;
import com.conflux.common.core.page.TableDataInfo;
import com.conflux.common.enums.BusinessType;
import com.conflux.common.utils.poi.ExcelUtil;
import com.conflux.web.controller.nft.domain.NftCollection;
import com.conflux.web.controller.nft.service.INftCollectionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * nft上链Controller
 * 
 * @author conflux
 * @date 2022-06-02
 */
@RestController
@RequestMapping("/system/collection")
public class NftCollectionController extends BaseController
{
    @Autowired
    private INftCollectionService nftCollectionService;

    /**
     * 查询nft上链列表
     */
    @PreAuthorize("@ss.hasPermi('system:collection:list')")
    @GetMapping("/list")
    public TableDataInfo list(NftCollection nftCollection)
    {
        startPage();
        List<NftCollection> list = nftCollectionService.selectNftCollectionList(nftCollection);
        return getDataTable(list);
    }

    /**
     * 导出nft上链列表
     */
    @PreAuthorize("@ss.hasPermi('system:collection:export')")
    @Log(title = "nft上链", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, NftCollection nftCollection)
    {
        List<NftCollection> list = nftCollectionService.selectNftCollectionList(nftCollection);
        ExcelUtil<NftCollection> util = new ExcelUtil<NftCollection>(NftCollection.class);
        util.exportExcel(response, list, "nft上链数据");
    }

    /**
     * 获取nft上链详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:collection:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(nftCollectionService.selectNftCollectionById(id));
    }

    /**
     * 新增nft上链
     */
    @PreAuthorize("@ss.hasPermi('system:collection:add')")
    @Log(title = "nft上链", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody NftCollection nftCollection)
    {
        return toAjax(nftCollectionService.insertNftCollection(nftCollection));
    }

    /**
     * 修改nft上链
     */
    @PreAuthorize("@ss.hasPermi('system:collection:edit')")
    @Log(title = "nft上链", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody NftCollection nftCollection)
    {
        return toAjax(nftCollectionService.updateNftCollection(nftCollection));
    }

    /**
     * 删除nft上链
     */
    @PreAuthorize("@ss.hasPermi('system:collection:remove')")
    @Log(title = "nft上链", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(nftCollectionService.deleteNftCollectionByIds(ids));
    }
}
