package com.conflux.web.controller.nft.controller;


import com.conflux.common.annotation.Log;
import com.conflux.common.core.controller.BaseController;
import com.conflux.common.core.domain.AjaxResult;
import com.conflux.common.core.page.TableDataInfo;
import com.conflux.common.enums.BusinessType;
import com.conflux.common.utils.poi.ExcelUtil;
import com.conflux.web.controller.nft.domain.NftLog;
import com.conflux.web.controller.nft.service.INftLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * nft日志Controller
 * 
 * @author conflux
 * @date 2022-06-02
 */
@RestController
@RequestMapping("/system/log")
public class NftLogController extends BaseController
{
    @Autowired
    private INftLogService nftLogService;

    /**
     * 查询nft日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:log:list')")
    @GetMapping("/list")
    public TableDataInfo list(NftLog nftLog)
    {
        startPage();
        List<NftLog> list = nftLogService.selectNftLogList(nftLog);
        return getDataTable(list);
    }

    /**
     * 导出nft日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:log:export')")
    @Log(title = "nft日志", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, NftLog nftLog)
    {
        List<NftLog> list = nftLogService.selectNftLogList(nftLog);
        ExcelUtil<NftLog> util = new ExcelUtil<NftLog>(NftLog.class);
        util.exportExcel(response, list, "nft日志数据");
    }

    /**
     * 获取nft日志详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:log:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(nftLogService.selectNftLogById(id));
    }

    /**
     * 新增nft日志
     */
    @PreAuthorize("@ss.hasPermi('system:log:add')")
    @Log(title = "nft日志", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody NftLog nftLog)
    {
        return toAjax(nftLogService.insertNftLog(nftLog));
    }

    /**
     * 修改nft日志
     */
    @PreAuthorize("@ss.hasPermi('system:log:edit')")
    @Log(title = "nft日志", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody NftLog nftLog)
    {
        return toAjax(nftLogService.updateNftLog(nftLog));
    }

    /**
     * 删除nft日志
     */
    @PreAuthorize("@ss.hasPermi('system:log:remove')")
    @Log(title = "nft日志", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(nftLogService.deleteNftLogByIds(ids));
    }
}
