package com.conflux.web.controller.nft.domain;

import com.google.gson.Gson;
import conflux.web3j.Account;
import conflux.web3j.Cfx;
import conflux.web3j.RpcException;
import conflux.web3j.request.Call;
import conflux.web3j.response.UsedGasAndCollateral;
import org.web3j.abi.FunctionEncoder;
import org.web3j.abi.TypeReference;
import org.web3j.abi.datatypes.Address;
import org.web3j.abi.datatypes.Type;
import org.web3j.abi.datatypes.generated.Uint256;

import java.math.BigInteger;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * @Author _007long
 * @Date 2022 05 31
 **/
public class ConfluxExecutor {
    private Account account;
    private String contract;

    public ConfluxExecutor(Account account, String contractAddress) throws RpcException {
        this.account = account;
        this.contract = contractAddress;
    }

    public String mulMint(Account.Option option, String address, List<BigInteger> _tokenIds) throws Exception {
        //String call(Account.Option option, Address contract, String method, Type<?>... inputs)
        return this.account.call(option,
                new conflux.web3j.types.Address(this.contract),
                "mulMint",
                new Address(address),
                new org.web3j.abi.datatypes.DynamicArray<>(
                        Uint256.class,
                        org.web3j.abi.Utils.typeMap(_tokenIds, Uint256.class)));
    }

    /**
     * 估算gas limit storage 出来
     *
     * @param cfx             cfx
     * @param account         account
     * @param contractAddress contractAddress
     * @return re
     */
    public UsedGasAndCollateral getEstimate(Cfx cfx, conflux.web3j.types.Address account,
                                            conflux.web3j.types.Address contractAddress,
                                            String address,
                                            List<BigInteger> _tokenIds) {
        Call call = new Call();
        call.setFrom(account);
        call.setTo(contractAddress);
        org.web3j.abi.datatypes.Function function = new org.web3j.abi.datatypes.Function(
                "mulMint",
                Arrays.<Type>asList(new Address(160, address),
                        new org.web3j.abi.datatypes.DynamicArray<Uint256>(
                                Uint256.class,
                                org.web3j.abi.Utils.typeMap(_tokenIds, Uint256.class))),
                Collections.<TypeReference<?>>emptyList());
        String data = FunctionEncoder.encode(function);
        System.out.println("============");
        System.out.println(data);
        call.setData(data);
        System.out.println(new Gson().toJson(call));
        return cfx.estimateGasAndCollateral(call).sendAndGet();
    }

}
