package com.conflux.web.controller.nft.service.impl;

import cn.hutool.core.date.DateUtil;
import com.conflux.common.core.domain.AjaxResult;
import com.conflux.common.utils.DateUtils;
import com.conflux.common.utils.StringUtils;
import com.conflux.common.utils.sign.Md5Utils;
import com.conflux.common.utils.uuid.IdUtils;
import com.conflux.web.controller.collect.domain.CollectConfig;
import com.conflux.web.controller.collect.service.ICollectConfigService;
import com.conflux.web.controller.contract.domain.ContractConfig;
import com.conflux.web.controller.contract.service.IContractConfigService;
import com.conflux.web.controller.nft.domain.*;
import com.conflux.web.controller.nft.domain.vo.FilterMapVO;
import com.conflux.web.controller.nft.service.ConfluxService;
import com.conflux.web.controller.nft.service.IHandlerStrategy;
import com.conflux.web.controller.nft.service.INftCollectionService;
import com.conflux.web.controller.nft.service.INftLogService;
import com.conflux.web.controller.privateKey.domain.PrivateKey;
import com.conflux.web.controller.privateKey.service.IPrivateKeyService;
import com.conflux.web.controller.util.AESUtil;
import com.conflux.web.controller.util.AddressUtil;
import com.conflux.web.controller.util.CfxUtils;
import com.conflux.web.controller.util.redis.RedisUtils;
import com.google.gson.Gson;
import conflux.web3j.Account;
import conflux.web3j.Cfx;
import conflux.web3j.Request;
import conflux.web3j.request.Epoch;
import conflux.web3j.request.LogFilter;
import conflux.web3j.response.Log;
import conflux.web3j.response.UsedGasAndCollateral;
import conflux.web3j.types.Address;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author _007long
 * @Date 2022 05 31
 **/
@Service
@Slf4j
public class ConfluxServiceImpl implements ConfluxService {
    @Resource
    private RedisUtils redisUtils;

    @Autowired
    private ICollectConfigService collectConfigService;
    @Autowired
    private IContractConfigService contractConfigService;
    @Autowired
    private INftCollectionService nftCollectionService;
    @Autowired
    private INftLogService nftLogService;
    @Resource
    private IHandlerStrategy iHandlerStrategy;
    @Autowired
    private ThreadPoolTaskExecutor executor;
    @Autowired
    private IPrivateKeyService privateKeyService;

    @SneakyThrows
    @Override
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult confluxPush(CheckArgs checkArgs) {
        //判断参数
        if (null == checkArgs) {
            return AjaxResult.error("参数为空！");
        }
        //判断必传参数
        if (StringUtils.isEmpty(checkArgs.getToken()) || StringUtils.isEmpty(checkArgs.getIds()) ||
                StringUtils.isEmpty(checkArgs.getSign()) || StringUtils.isEmpty(checkArgs.getTimesTamp()) ||
                StringUtils.isEmpty(checkArgs.getUnitName()) || StringUtils.isEmpty(checkArgs.getAppId()) ||
                StringUtils.isEmpty(checkArgs.getPrivateKey())) {
            return AjaxResult.error("参数缺失！");
        }
        ContractConfig contractConfig = contractConfigService.selectContractConfigByUnitName(checkArgs.getUnitName());
        if (null == contractConfig) {
            return AjaxResult.error("合约不存在！");
        }
        if (checkArgs.getIds().length > contractConfig.getThreshold()) {
            return AjaxResult.error("单次上链数量超出上限！");
        }
        int sum = nftLogService.selectSum(checkArgs.getUnitName());
        if (sum>contractConfig.getTotalThreshold()){
            return AjaxResult.error("上链总数量超出上限！");
        }
        String[] ids = checkArgs.getIds();
        List<NftCollection> nftCollections = nftCollectionService.selectNftCollectionListByIds(ids, null);
        if (nftCollections.size() > 0) {
            List list = new ArrayList();
            for (NftCollection n : nftCollections) {
                list.add(n.getInformationTableId());
            }
            return AjaxResult.error("已经上链的id", list);
        }
        String checkArgsToken = AESUtil.Decrypt(checkArgs.getToken(), ConfluxArt.AESKEY);
        String token = (String) redisUtils.get(checkArgs.getAppId());//通过私钥获取缓存token
        PrivateKey privateKey = privateKeyService.selectPrivateKeyByAppId(checkArgs.getAppId());
        if (null==privateKey){
            return AjaxResult.error("appId错误！");
        }
        String md5Key = Md5Utils.md5Salt(privateKey.getPrivateKey());
        if(!checkArgs.getPrivateKey().equals(md5Key)){
            return AjaxResult.error("privateKey私钥出错！");
        }
        String redisSign = (String) redisUtils.get(checkArgs.getPrivateKey());//通过密钥获取sign签名
        if (null == token) {
            return AjaxResult.error("token已过期！");
        }
        String sign = AESUtil.Decrypt(checkArgs.getSign(), ConfluxArt.AESKEY);
        //校验token和sign签名
        if (token.equals(checkArgsToken) && sign.equals(redisSign)) {
            //校验时间戳
            long time = Long.valueOf(checkArgs.getTimesTamp());
            long timesTamp = System.currentTimeMillis();
            if (timesTamp - time > 30000l) {
                return AjaxResult.error("时间参数错误！");
            }
            //判断上链的数量
            CollectConfig collectConfig = collectConfigService.selectCollectConfigByStatus();
            if (collectConfig.getMintPause()) {
                log.info("[--------------------->mintNft][start pause]");
                Thread.sleep(5000);
                return AjaxResult.error("pause参数异常！");
            }
            Cfx cfx = Cfx.create(collectConfig.getNode(), 3, 1000);

            Account account = Account.create(cfx, AESUtil.decrypt(contractConfig.getPrivateKey()));
            Account.Option opt = new Account.Option();
            opt.withValue(BigInteger.ZERO);
            opt.withChainId(collectConfig.getChainId());
            opt.withEpochHeight(cfx.getEpochNumber().sendAndGet());
            opt.withGasPrice(contractConfig.getGasPrice());
            //获取余额
            BigInteger balance = cfx.getBalance(account.getAddress()).sendAndGet();
            log.info("[--------------------->mintNft][balance]{}", balance);
            if (balance.compareTo(new BigInteger("50000000000000000000")) < 0) {
                log.info("[--------------------->mintNft][balance not enough 50]{}", balance);
                //sms notice
                return AjaxResult.error("账户余额不足！");
            }
            String contract = contractConfig.getContract();
            String owner = contractConfig.getOwner();
            ConfluxExecutor confluxExecutor = new ConfluxExecutor(account, contract);
            List<String> list = Arrays.asList(checkArgs.getIds());
            long start = DateUtil.currentSeconds();
            log.info("[--------------------->mintNft][time]{}", new SimpleDateFormat("HH:mm:ss").format(new Date(start * 1000)));
            List<BigInteger> tokenIds = list.stream().map(n -> new BigInteger(n, 16)).collect(Collectors.toList());
            log.info("[--------------------->mintNft][tokenIds]:{}", tokenIds);
            UsedGasAndCollateral est = confluxExecutor.getEstimate(cfx, new Address(owner),
                    new Address(contract), AddressUtil.getHexAddress(owner), tokenIds);
            log.info("[--------------------->mintNft][GasUsed]{}", est.getGasUsed());
            log.info("[--------------------->mintNft][StorageCollateralized]{}", est.getStorageCollateralized());
            opt.withGasLimit(est.getGasUsed());
            opt.withStorageLimit(est.getStorageCollateralized());
            String hash = confluxExecutor.mulMint(opt,
                    AddressUtil.getHexAddress(owner), tokenIds
            );
            log.info("[--------------------->mintNft][hash],{}", hash);
            log.info("[--------------------->mintNft][time end]{}", +(DateUtil.currentSeconds() - start));
            //保存上链的id
            nftCollectionService.insertListNftCollection(contractConfig.getUnitName(), list);
            //保存日志记录
            NftLog nftLog = new NftLog();
            nftLog.setUnitName(contractConfig.getUnitName());
            nftLog.setCreateTime(new Date());
            nftLog.setNftNum(checkArgs.getIds().length);
            nftLogService.insertNftLog(nftLog);
            log.info("上链日志----->:{" + contractConfig.getUnitName() + "}{" + new Date() + "}{" + checkArgs.getIds().length + "}");
            //使用线程铸造
            executor.execute(() -> {
                boolean isFalg = true;
                while (isFalg) {
                    try {
                        Thread.sleep(500);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    if (executeMakeUp(contract, contractConfig.getUnitName())) {
                        isFalg = false;
                    }
                    System.out.println("--------正在铸造----------->");
                }
            });
            Map<String,Object> map=new HashMap<>();
            map.put("contract",contract);
//            map.put("ids",tokenIds);
            return AjaxResult.success("上链成功！",map);
        }
        return AjaxResult.error("token或sign签名失效！");
    }

    @Override
    public AjaxResult getToken(PrivateKey privateKey) {
        if (null == privateKey.getAppId() || null == privateKey.getPrivateKey()) {
            return AjaxResult.error("参数缺失！");
        }
        PrivateKey privateKey1 = privateKeyService.selectPrivateKeyByAppId(privateKey.getAppId());
        if (null==privateKey1){
            return AjaxResult.error("appId错误！");
        }
        String md5Key = Md5Utils.md5Salt(privateKey1.getPrivateKey());
        if (!privateKey.getPrivateKey().equals(md5Key)){
            return AjaxResult.error("privateKey错误！");
        }
        Map<String, Object> map = new HashMap<>();
        String token = IdUtils.fastSimpleUUID();
        long time = System.currentTimeMillis();
        map.put("token", token);
        map.put("timesTamp", time);
        try {
            map.put("sign", AESUtil.encrypt(ConfluxArt.SIGN + time));
            redisUtils.set(privateKey.getAppId(), token,3600l);//缓存一小时
            redisUtils.set(md5Key, AESUtil.encrypt(ConfluxArt.SIGN + time),3600l);
            map.put("expireTime",3600);//过期时间 单位秒
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.success(map);
    }

    @Override
    public AjaxResult getNftStatus(CheckArgs checkArgs) {
        if(StringUtils.isEmpty(checkArgs.getIds())||StringUtils.isEmpty(checkArgs.getAppId())||StringUtils.isEmpty(checkArgs.getPrivateKey())){
            return AjaxResult.error("参数缺失！");
        }
        PrivateKey privateKey = privateKeyService.selectPrivateKeyByAppId(checkArgs.getAppId());
        if (null==privateKey){
            return AjaxResult.error("appId错误！");
        }
        String md5Key = Md5Utils.md5Salt(privateKey.getPrivateKey());
        if(!checkArgs.getPrivateKey().equals(md5Key)){
            return AjaxResult.error("privateKey私钥出错！");
        }
        List<NftCollection> nftCollections = nftCollectionService.selectNftCollectionListByIds(checkArgs.getIds(), 2);
        if (nftCollections.size() > 0) {
            List<Map> list = new ArrayList();
            for (NftCollection n : nftCollections) {
                Map<String,Object> map=new HashMap<>();
                map.put("informationTableId",n.getInformationTableId());
                map.put("successTime", DateUtils.dateToStr(n.getUpdatedTime()));
                list.add(map);
            }
            return AjaxResult.success("铸造成功的id", list);
        }
        return AjaxResult.error("暂无数据");
    }

    public boolean executeMakeUp(String contract, String markName) {
        log.info("[dispatchHandler]{}", "开始监听数据");
        CollectConfig collect = (CollectConfig) redisUtils.get(contract);
        if (null == collect) {
            collect = collectConfigService.selectCollectConfigByStatus();
            redisUtils.set(contract, collect);
        }
        if (collect.getOnPause()) {
            log.info("[dispatchHandler]{}", "暂停中");
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            return false;
        }
        //=======检查数据=======
        List<EventParam> eventParams = new ArrayList<>();
        EventParam eventParam = new EventParam();
        eventParam.setChainId(collect.getChainId());
        eventParam.setContract(contract);
        eventParam.setParam("address,address,uint256");
        eventParam.setMethod("Transfer");
        eventParam.setType(20000);
        eventParam.setRemark(markName);
        eventParams.add(eventParam);
        if (CollectionUtils.isEmpty(eventParams)) {
            log.error("[dispatchHandler]{}", "没有需要监听的数据");
            return false;
        }

        Gson gson = new Gson();
        //获取event，使用 webService
        //记录数据
        Cfx cfx = null;
        try {
            cfx = Cfx.create(collect.getNode(), 3, 1000);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        FilterMapVO vo = getFilter(eventParams);
        //获取数据高度
        String keyNumber = contract + "number";
        long from = 0l;
        if (null == redisUtils.get(keyNumber)) {
            from = collect.getEpochNumber();
        } else {
            Integer number = (Integer) redisUtils.get(keyNumber);
            from = Long.valueOf(number);
        }
        // Invoke cfx method
        BigInteger epoch = cfx.getEpochNumber().sendAndGet();
        log.info("[executeMakeUp][Current epoch]:{}", epoch);
        long to = epoch.intValue() - 5;
        if (to - from > collect.getLimitCount()) {
            to = from + collect.getLimitCount();
        }
        vo.getLogFilter().setFromEpoch(Epoch.numberOf(from));
        //当前高度
        vo.getLogFilter().setToEpoch(Epoch.numberOf(to));
        Request<List<Log>, Log.Response> events = cfx.getLogs(vo.getLogFilter());
        try {
            events = cfx.getLogs(vo.getLogFilter());
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (events == null) {
            log.info("[executeMakeUp][请重启服务]");
            return false;
        }
        List<Log> logList = events.sendAndGet();
        boolean isSuccess = false;
        if (CollectionUtils.isEmpty(logList)) {
            log.info("[executeMakeUp][此区间暂无日志]:from:{} to:{}", from, to);
            isSuccess = true;
        }
        for (Log l : logList) {
            //处理高度信息
            String evAddress = l.getTopics().get(0);
            String tokenAddress = l.getAddress().getAddress();
            if (tokenAddress.contains("TYPE")) {
                Address a = new Address(tokenAddress);
                tokenAddress = a.getAddress();
            }
            int type = vo.getMap().get(evAddress + "-" + tokenAddress.toLowerCase());
            if (type == 0) {
                log.info("[dispatchHandler][暂无符合条件的数据]");
                return false;
            }
            boolean res;
            try {
                res = iHandlerStrategy.relayHandler(cfx, gson.toJson(l), type);
            } catch (Exception e) {
                e.printStackTrace();
                isSuccess = false;
                log.info("[dispatchHandler][数据错误未分配]{}", l.getData());
                break;
            }
            if (res) {
                log.info("[dispatchHandler][分配成功]{}", evAddress);
                isSuccess = true;
            } else {
                log.info("[dispatchHandler][数据错误未分配]{}", l.getData());
                isSuccess = false;
                break;
            }
        }
        //分析数据拉取成功
        CollectConfig t = new CollectConfig();
        t.setId(collect.getId());
        t.setUpdateTime(DateUtil.currentSeconds());
        if (isSuccess) {
            t.setEpochNumber(to);
            log.info("[dispatchHandler][更新高度]:{}", to);
            redisUtils.set(keyNumber, to);
        }
        if (logList.size() > 0) {
            int i = collectConfigService.updateCollectConfig(t);
            if (i <= 0) {
                log.error("[dispatchHandler][更新高度失败]:{}", to);
                throw new RuntimeException("更新高度失败");
            }
            log.info("[dispatchHandler][更新高度成功]:{}", to);
            redisUtils.del(contract);
            redisUtils.del(keyNumber);
            return true;
        }
        return false;
    }

    private FilterMapVO getFilter(List<EventParam> eventParams) {
        // PubSub Subscribe to incoming events and process incoming events
        LogFilter filter = new LogFilter();
        //添加监听合约：
        List<Address> moList = eventParams.stream().map(
                eventParam -> new Address(eventParam.getContract())).collect(Collectors.toList());
        filter.setAddress(moList);
        //添加监听方法
        List<String> list = new ArrayList<>();
        //Transfer
        Map<String, Integer> map = new HashMap<>(eventParams.size());
        eventParams.forEach(eventParam -> {
            String evAddress = CfxUtils.getEventKeccak(eventParam.getMethod(), Arrays.asList(eventParam.getParam().split(",")));
//            log.info("token:{} method:{} kec:{}", eventParam.getContract(), eventParam.getMethod(), evAddress);
            map.put(evAddress + "-" + eventParam.getContract().toLowerCase(), eventParam.getType());
            list.add(evAddress);
        });
        List<List<String>> lists = new ArrayList<>();
        lists.add(list);
        filter.setTopics(lists);
        FilterMapVO vo = new FilterMapVO();
        vo.setLogFilter(filter);
        vo.setMap(map);
        return vo;
    }
}
