package com.conflux.web.controller.privateKey.service.impl;

import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import com.conflux.common.utils.DateUtils;
import com.conflux.common.utils.uuid.IdUtils;
import com.conflux.web.controller.nft.domain.ConfluxArt;
import com.conflux.web.controller.util.AESUtil;
import com.conflux.web.controller.util.redis.RedisUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import com.conflux.web.controller.privateKey.mapper.PrivateKeyMapper;
import com.conflux.web.controller.privateKey.domain.PrivateKey;
import com.conflux.web.controller.privateKey.service.IPrivateKeyService;

import javax.annotation.Resource;

/**
 * 私钥申请Service业务层处理
 * 
 * @author conflux
 * @date 2022-06-07
 */
@Service
public class PrivateKeyServiceImpl implements IPrivateKeyService 
{
    @Autowired
    private PrivateKeyMapper privateKeyMapper;
    @Resource
    private RedisUtils redisUtils;
    @Resource
    RedisTemplate redisTemplate;
    /**
     * 查询私钥申请
     * 
     * @param id 私钥申请主键
     * @return 私钥申请
     */
    @Override
    public PrivateKey selectPrivateKeyById(Long id)
    {
        return privateKeyMapper.selectPrivateKeyById(id);
    }

    @Override
    public PrivateKey selectPrivateKeyByAppId(String appId) {
        return privateKeyMapper.selectPrivateKeyByAppId(appId);
    }

    /**
     * 查询私钥申请列表
     * 
     * @param privateKey 私钥申请
     * @return 私钥申请
     */
    @Override
    public List<PrivateKey> selectPrivateKeyList(PrivateKey privateKey)
    {
        return privateKeyMapper.selectPrivateKeyList(privateKey);
    }

    /**
     * 新增私钥申请
     * 
     * @param privateKey 私钥申请
     * @return 结果
     */
    @Override
    public int insertPrivateKey(PrivateKey privateKey)  {
        privateKey.setCreateTime(DateUtils.getNowDate());
        String fastUUID = IdUtils.fastUUID();
        String appid = null;
        String key= null;
        try {
            appid =generateOrderSn();
            key = AESUtil.encrypt(fastUUID, ConfluxArt.AESKEY);
        } catch (Exception e) {
            e.printStackTrace();
        }
        privateKey.setAppId(appid);
        privateKey.setPrivateKey(key.toLowerCase());
        return privateKeyMapper.insertPrivateKey(privateKey);
    }

    /**
     * 修改私钥申请
     * 
     * @param privateKey 私钥申请
     * @return 结果
     */
    @Override
    public int updatePrivateKey(PrivateKey privateKey)
    {
        return privateKeyMapper.updatePrivateKey(privateKey);
    }

    /**
     * 批量删除私钥申请
     * 
     * @param ids 需要删除的私钥申请主键
     * @return 结果
     */
    @Override
    public int deletePrivateKeyByIds(Long[] ids)
    {
        return privateKeyMapper.deletePrivateKeyByIds(ids);
    }

    /**
     * 删除私钥申请信息
     * 
     * @param id 私钥申请主键
     * @return 结果
     */
    @Override
    public int deletePrivateKeyById(Long id)
    {
        return privateKeyMapper.deletePrivateKeyById(id);
    }

    /**
     * 生成appid
     * @return
     */
    public String generateOrderSn() {
        StringBuilder sb = new StringBuilder();
        String date = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
        // 拼接key值
        String key = "UUID:"+ date;
        int code = (int)(Math.random() * (40000 -10000)) + 10000;
        Long increment = incr(key, code);
        sb.append(date);
        sb.append(String.format("%02d", 2));
        sb.append(String.format("%02d",2));
        String incrementStr = increment.toString();
        if (incrementStr.length() <= 6) {
            sb.append(String.format("%06d", increment));
        } else {
            sb.append(incrementStr);
        }
        return sb.toString();
    }

    /***
     * 按delta递增
     * @param key
     * @param delta
     * @return
     */
    public Long incr(String key, long delta) {
        return redisTemplate.opsForValue().increment(key, delta);
    }
}
